@@ -9,6 +9,7 @@ import speakeasy.winenv.defs.windows.advapi32 as adv32
 import speakeasy.windows.objman as objman
 
 from .. import api
+from Crypto.Cipher import ARC4
 import hashlib
 
 SERVICE_STATUS_HANDLE_BASE = 0x1000
@@ -35,6 +36,8 @@ class AdvApi32(api.ApiHandler):
         self.curr_handle = 0x2800
         self.service_status_handle = SERVICE_STATUS_HANDLE_BASE
 
+        self.rc4 = None
+
         super(AdvApi32, self).__get_hook_attrs__(self)
 
     def get_handle(self):
@@ -229,8 +232,30 @@ class AdvApi32(api.ApiHandler):
 
         hKey, dwIndex, lpName, cchName = argv
 
-        cw = self.get_char_width(ctx)
+        _argv = argv + [0, 0, 0, 0]
+        rv = self.RegEnumKeyEx(emu, _argv, ctx)
+        argv[:] = _argv[: 4]
 
+        return rv
+
+    @apihook('RegEnumKeyEx', argc=8, conv=_arch.CALL_CONV_STDCALL)
+    def RegEnumKeyEx(self, emu, argv, ctx={}):
+        '''
+        LSTATUS RegEnumKeyEx(
+            HKEY      hKey,
+            DWORD     dwIndex,
+            LPSTR     lpName,
+            LPDWORD   lpcchName,
+            LPDWORD   lpReserved,
+            LPSTR     lpClass,
+            LPDWORD   lpcchClass,
+            PFILETIME lpftLastWriteTime
+        );
+        '''
+
+        hKey, dwIndex, lpName, cchName, res, pcls, cchcls, last_write = argv
+
+        cw = self.get_char_width(ctx)
         rv = windefs.ERROR_INVALID_HANDLE
         if hKey:
             key = self.reg_get_key(hKey)
@@ -251,7 +276,37 @@ class AdvApi32(api.ApiHandler):
                             name = name.encode('utf-8')
                         self.mem_write(lpName, name)
                         rv = windefs.ERROR_SUCCESS
+            self.log_registry_access(key.get_path(), "list_subkeys")
+        return rv
 
+    @apihook('RegCreateKey', argc=3)
+    def RegCreateKey(self, emu, argv, ctx={}):
+        """
+        LSTATUS RegCreateKey(
+            HKEY    hKey,
+            LPCWSTR lpSubKey,
+            PHKEY   phkResult
+        );
+        """
+        hkey, lpSubKey, phkResult = argv
+        rv = windefs.ERROR_INVALID_HANDLE
+        if hkey:
+            key = self.reg_get_key(hkey)
+            argv[0] = key.get_path()
+            if not key:
+                rv = windefs.ERROR_INVALID_HANDLE
+            else:
+                cw = self.get_char_width(ctx)
+                if lpSubKey:
+                    lpSubKey = self.read_mem_string(lpSubKey, cw)
+                    argv[1] = lpSubKey
+                    sub_key_path = key.get_path() + '\\' + lpSubKey
+                    self.emu.reg_create_key(sub_key_path)
+                    self.log_registry_access(sub_key_path, "create_key")
+                else:
+                    hkey = (hkey).to_bytes(self.get_ptr_size(), 'little')
+                    self.mem_write(phkResult, hkey)
+                    rv = windefs.ERROR_SUCCESS
         return rv
 
     @apihook('RegQueryInfoKey', argc=12, conv=_arch.CALL_CONV_STDCALL)
@@ -482,6 +537,22 @@ class AdvApi32(api.ApiHandler):
 
         return 0x1
 
+    @apihook('RevertToSelf', argc=0)
+    def RevertToSelf(self, emu, argv, ctx={}):
+        '''
+        BOOL RevertToSelf();
+        '''
+        return 1
+
+    @apihook('ImpersonateLoggedOnUser', argc=1)
+    def ImpersonateLoggedOnUser(self, emu, argv, ctx={}):
+        '''
+        BOOL ImpersonateLoggedOnUser(
+        HANDLE hToken
+        );
+        '''
+        return 1
+
     @apihook('OpenSCManager', argc=3)
     def OpenSCManager(self, emu, argv, ctx={}):
         '''
@@ -590,6 +661,24 @@ class AdvApi32(api.ApiHandler):
 
         return rv
 
+    @apihook('SystemFunction036', argc=2)
+    def RtlGenRandom(self, emu, argv, ctx={}):
+        '''
+        BOOLEAN RtlGenRandom(
+            PVOID RandomBuffer,
+            ULONG RandomBufferLength
+        );
+        '''
+        RandomBuffer, RandomBufferLength = argv
+
+        rv = False
+        if RandomBuffer and RandomBufferLength:
+            buf = bytes([i for i in range(RandomBufferLength)])
+            self.mem_write(RandomBuffer, buf)
+            rv = True
+
+        return rv
+
     @apihook('CryptAcquireContext', argc=5)
     def CryptAcquireContext(self, emu, argv, ctx={}):
         '''
@@ -614,7 +703,7 @@ class AdvApi32(api.ApiHandler):
             argv[2] = prov_str
 
         cm = emu.get_crypt_manager()
-        hnd = cm.crypt_open(cont_str, prov_str, dwProvType, dwFlags)
+        hnd = cm.crypt_open(cname=cont_str, pname=prov_str, ptype=dwProvType, flags=dwFlags)
 
         if hnd and phProv:
             self.mem_write(phProv, hnd.to_bytes(emu.get_ptr_size(), 'little'))
@@ -731,6 +820,7 @@ class AdvApi32(api.ApiHandler):
 
         user = emu.get_user()
         user_name = user.get('name')
+        argv[0] = user_name
 
         if lpBuffer:
             if cw == 2:
@@ -800,7 +890,7 @@ class AdvApi32(api.ApiHandler):
         if not info_len:
             rv = False
             emu.set_last_error(windefs.ERROR_INSUFFICIENT_BUFFER)
-            
+
         if info_class == 20 and info and emu.get_user().get('is_admin', True):
             self.mem_write(info, (1).to_bytes(4, 'little'))
 
@@ -952,7 +1042,7 @@ class AdvApi32(api.ApiHandler):
         }
 
         hProv, Algid, hKey, dwFlags, phHash = argv
-        argv[1] = hash_algs.get(Algid, Algid)
+        argv[1] = hash_algs.get(Algid, Algid)[0]
 
         if hKey != 0:
             return 0
@@ -990,8 +1080,137 @@ class AdvApi32(api.ApiHandler):
         hnd.update(data)
         return 1
 
-    @apihook('RegGetValueW', argc=7, conv=_arch.CALL_CONV_STDCALL)
-    def RegGetValueW(self, emu, argv, ctx={}):
+    @apihook('CryptGetHashParam', argc=5)
+    def CryptGetHashParam(self, emu, argv, ctx={}):
+        '''
+        BOOL CryptGetHashParam(
+          HCRYPTHASH hHash,
+          DWORD      dwParam,
+          BYTE       *pbData,
+          DWORD      *pdwDataLen,
+          DWORD      dwFlags
+        );
+        '''
+        hHash, dwParam, pbData, pdwDataLen, dwFlags = argv
+
+        param_enums = {
+            1: "HP_ALGID",
+            2: "HP_HASHVAL",
+            4: "HP_HASHSIZE",
+            5: "HP_HMAC_INFO"
+        }
+
+        if dwParam in param_enums.keys():
+            argv[1] = param_enums[dwParam]
+
+        return 1
+
+    @apihook('CryptDestroyHash', argc=1)
+    def CryptDestroyHash(self, emu, argv, ctx={}):
+        """
+        BOOL CryptDestroyHash(
+          HCRYPTHASH hHash
+        );
+        """
+        hHash = argv
+
+        return 1
+
+    @apihook('CryptDeriveKey', argc=5)
+    def CryptDeriveKey(self, emu, argv, ctx={}):
+        """
+        BOOL CryptDeriveKey(
+          HCRYPTPROV hProv,
+          ALG_ID     Algid,
+          HCRYPTHASH hBaseData,
+          DWORD      dwFlags,
+          HCRYPTKEY  *phKey
+        );
+        """
+
+        hProv, Algid, hBaseData, dwFlags, phKey = argv
+
+        # Only RC4 supported right now
+        if Algid != 0x6801:
+            return 0
+
+        hnd = self.hash_objects.get(hBaseData, None)
+
+        if hnd is None:
+            emu.set_last_error(windefs.ERROR_INVALID_HANDLE)
+            return 0
+
+        # CryptDeriveKey zeroes out the last 11 bytes of the hash,
+        # so we gotta do the same before it is written to the
+        # phKey structure
+        fixed_digest = hnd.digest()[:5] + b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
+
+        ptrsz = emu.get_ptr_size()
+
+        hKey = self.win.HCRYPTKEY(ptrsz)
+        hKey.Algid = Algid
+        hKey.keylen = hnd.digest_size
+        hKey.keyp = self.mem_alloc(hKey.keylen)
+
+        hKeyp = self.mem_alloc(hKey.sizeof())
+
+        self.mem_write(hKey.keyp, fixed_digest)
+
+        self.mem_write(hKeyp, hKey.get_bytes())
+        self.mem_write(phKey, hKeyp.to_bytes(ptrsz, "little"))
+
+        return 1
+
+    @apihook('CryptDecrypt', argc=6)
+    def CryptDecrypt(self, emu, argv, ctx={}):
+        """
+        BOOL CryptDecrypt(
+          HCRYPTKEY  hKey,
+          HCRYPTHASH hHash,
+          BOOL       Final,
+          DWORD      dwFlags,
+          BYTE       *pbData,
+          DWORD      *pdwDataLen
+        );
+        """
+
+        hKey, hHash, Final, dwFlags, pbData, pdwDataLen = argv
+
+        # Hashing not supported
+        if hHash:
+            return 0
+
+        ptrsz = emu.get_ptr_size()
+
+        hKey = self.mem_cast(self.win.HCRYPTKEY(ptrsz), hKey)
+
+        # Only RC4 supported right now
+        if hKey.Algid != 0x6801:
+            return 0
+
+        encdatalen_b = self.mem_read(pdwDataLen, 4)
+        encdatalen = int.from_bytes(encdatalen_b, "little")
+
+        encdata = self.mem_read(pbData, encdatalen)
+
+        key = self.mem_read(hKey.keyp, hKey.keylen)
+
+        if self.rc4 is None:
+            self.rc4 = ARC4.new(key)
+
+        dec = self.rc4.decrypt(encdata)
+        declen = len(dec)
+
+        self.mem_write(pbData, dec)
+        self.mem_write(pdwDataLen, int.to_bytes(declen, 4, "little"))
+
+        if Final == True:
+            self.rc4 = None
+
+        return 1
+
+    @apihook('RegGetValue', argc=7, conv=_arch.CALL_CONV_STDCALL)
+    def RegGetValue(self, emu, argv, ctx={}):
         '''
         LSTATUS RegGetValueW(
             HKEY    hkey,
@@ -1050,3 +1269,46 @@ class AdvApi32(api.ApiHandler):
                                      buffer=lpData)
 
         return rv
+
+    @apihook('EnumServicesStatus', argc=8, conv=_arch.CALL_CONV_STDCALL)
+    def EnumServicesStatus(self, emu, argv, ctx={}):
+        '''
+        BOOL EnumServicesStatusA(
+          SC_HANDLE              hSCManager,
+          DWORD                  dwServiceType,
+          DWORD                  dwServiceState,
+          LPENUM_SERVICE_STATUSA lpServices,
+          DWORD                  cbBufSize,
+          LPDWORD                pcbBytesNeeded,
+          LPDWORD                lpServicesReturned,
+          LPDWORD                lpResumeHandle
+        );
+        '''
+        hSCManager, dwServiceType, dwServiceState, lpServices, cbBufSize, \
+            pcbBytesNeeded, lpServicesReturned, lpResumeHandle = argv
+
+        service_type_str = adv32.get_define_int(dwServiceType, 'SERVICE_')
+        if service_type_str:
+            argv[1] = service_type_str
+
+        service_state_str = adv32.get_define_int(dwServiceState, 'SERVICE_')
+        if service_state_str:
+            argv[2] = service_state_str
+
+        # TODO: Populate service status output
+        return 1
+
+    @apihook('OpenService', argc=3, conv=_arch.CALL_CONV_STDCALL)
+    def OpenService(self, emu, argv, ctx={}):
+        '''
+        SC_HANDLE OpenServiceA(
+          SC_HANDLE hSCManager,
+          LPCSTR    lpServiceName,
+          DWORD     dwDesiredAccess
+        );
+        '''
+        hSCManager, lpServiceName, dwDesiredAccess = argv
+        cw = self.get_char_width(ctx)
+        svcname = self.read_mem_string(lpServiceName, cw)
+        argv[1] = svcname
+        return self.get_handle()
